-- Daily Global Check
-- by Jadya
-- EU-Well of Eternity

local tinsert, tremove = table.insert, table.remove

local addonName, addonTable = ...


local L = addonTable.L
local C = addonTable.C
local ng = NyxGUI(C.NyxGUIVersion)

local AddColor = addonTable.AddColor

local constructors = DailyGlobalCheck.Constructors
local options      = DailyGlobalCheck.OptFunctions
local maps         = DailyGlobalCheck.MapFunctions
local lists        = DailyGlobalCheck.ListFunctions

local addontitle = "|TInterface\\icons\\INV_Sigil_Freya:16 |t|cff00AAFFDaily Global Check|r"
local dgcicon = "Interface\\icons\\INV_Sigil_Freya"

------ DGC Base Options ------
function options:OnOptionChanged(opt)
-- General
 if opt == "show_worldmapicons" then
  maps:UpdateIcons("worldmap")
 elseif opt == "show_minimapicons" then
  maps:UpdateIcons("minimap")
-- Map Icons UI
 elseif opt == "worldmap_icon_interact" then
  maps:UpdateInteractable("worldmap")
 elseif opt == "minimap_icon_interact" then
  maps:UpdateInteractable("minimap")
 elseif opt == "worldmap_icon_size" then
  maps:UpdateIconsSize("worldmap")
 elseif opt == "minimap_icon_size" then
  maps:UpdateIconsSize("minimap")
 end
end

function options.setup_quests_panel(f)
 local chkcounter = 0

 local function CreateCheckbox(opt, text)

  local c = ng:New(addonName, "Checkbox", f:GetName().."Checkbox"..chkcounter, f)
  c:SetText("|cffFFFFFF"..text.."|r")

  tinsert(f.checkboxes, c)
  c:SetPoint("TOPLEFT", f, "TOPLEFT", 10, -20 * chkcounter)
  c.text:SetWidth(C.modes[3].x - 40)
  c.text:SetJustifyH("LEFT")

  c:SetScript("OnClick", function(self)
    if opt == "minimap_icon" then
      DailyGlobalCheck_Options["minimap_icon"].hide = not self:GetChecked()
 	 addonTable.ldbicon:Refresh("DailyGlobalCheck_broker", DailyGlobalCheck_broker, DailyGlobalCheck_Options["minimap_icon"])
    else
     DailyGlobalCheck_Options[opt] = self:GetChecked()
	 options:OnOptionChanged(opt)
	 DailyGlobalCheck:Refresh()
    end
  end)

  if opt == "minimap_icon" then
   c:SetChecked(DailyGlobalCheck_Options["minimap_icon"] and not DailyGlobalCheck_Options["minimap_icon"].hide or false)
  else
   c:SetChecked(DailyGlobalCheck_Options[opt])
  end
  chkcounter = chkcounter + 1
  return c
 end

 f.checkboxes = {}
 CreateCheckbox("minimap_icon",L["opt_minimap"])
 chkcounter = chkcounter + 1
 CreateCheckbox("show_questtype", L["opt_questtype"])
 CreateCheckbox("show_prefix", L["opt_prefix"])
 CreateCheckbox("show_zone", L["opt_zonename"])
 CreateCheckbox("show_suffix", L["opt_suffix"])
 CreateCheckbox("show_coordinates", L["opt_coords"])
 CreateCheckbox("hide_completed", L["opt_hidecompleted"])
 chkcounter = chkcounter + 2
 CreateCheckbox("show_questlogbtn", L["opt_showquestbtn"])
end

------ Plugin Options ------
local lastselected
local chkcounter = 0

-- widgets handling
local widgets_pool = {}

local function getwidget(t)
 local panel = DGC_Mainframe.panels[3]
 local found
 
 for k,v in pairs(widgets_pool) do
  if v.t == t then
   found = k
   break
  end
 end

 if found then
  local f = widgets_pool[found]
  tremove(widgets_pool, found)
  tinsert(panel.widgets, f)
  return f
 elseif t == "Checkbox" then
  local f = ng:New(addonName, "Checkbox", panel:GetName().."Checkbox"..chkcounter, panel)
  chkcounter = chkcounter + 1
  tinsert(panel.widgets, f)
  f.t = t
  return f
 end
end
--

local function updatepluginpanel()
 local list = DailyGlobalCheck:GetSelectedList()
 local of = DGC_Mainframe.panels[C.MODE_OPTIONS_PLUGIN]

 if not of or list == lastselected then return end
 
  for _,v in pairs(of.widgets) do
   v:Hide()
   tinsert(widgets_pool, v)
  end
  wipe(of.widgets)

 if not list or not list["Options"] then
  --of.text:SetText(L["plugin_opt_none"])
  of.text:SetText("")
 else
  of.text:SetText("")

  -- widgets creation
  if not DailyGlobalCheck_PluginData[list["Title"]] then DailyGlobalCheck_PluginData[list["Title"]] = {} end
  local svar = DailyGlobalCheck_PluginData[list["Title"]]
  if not svar.options then svar.options = {} end

  for i,v in pairs(list["Options"]) do
   local t, text, refresh, flags = unpack(v)
   local f = getwidget(t)
   f.tag = i
   f:SetPoint("TOPLEFT", of, "TOPLEFT", 10, -20 * i)

   -- Checkbox
   if t == "Checkbox" then
    f.text:SetText("|cffFFFFFF"..text.."|r")
    f:SetChecked(svar.options[i])

    f:SetScript("OnClick", function(self)

     if not svar.options then svar.options = {} end
	 svar.options[self.tag] = f:GetChecked()
	 
	 if list.OnOptionChanged then
	  list.OnOptionChanged(self.tag)
	 end
	 
	 if refresh then
	  DailyGlobalCheck:Refresh()
     end
    end)
   end
   f:Show()
  end
 end
 
 if list then
  local svar = lists:GetSvar(list)
  if svar.show_mapicons_worldmap == nil then 
   lists:setShowMapIcons(list, "worldmap", true)
  end
  if svar.show_mapicons_minimap == nil then 
   lists:setShowMapIcons(list, "minimap", true)
  end
  of.showworldmapicons:SetChecked(lists:getShowMapIcons(list, "worldmap"))
  of.showminimapicons:SetChecked(lists:getShowMapIcons(list, "minimap"))
 end

 lastselected = list
end

function options.setup_plugin_panel(p)

 local t = p:CreateFontString(nil, "OVERLAY", "GameTooltipText")
 t:SetPoint("TOPLEFT", p, "TOPLEFT", 10, -10)
 t:SetPoint("BOTTOMRIGHT", p, "TOPRIGHT", -10, -80)
 t:SetWordWrap(true)
 p.text = t
 
 p.widgets = {}

 local f = ng:New(addonName, "Checkbox", "DGCOptionsPluginWMCheckbox", p)
 f:SetText("|cffFFFFFF"..L["plugin_mapicons"].."|r")
 f:SetPoint("BOTTOMLEFT", p, "BOTTOMLEFT", 10, 50)
 f:SetScript("OnClick", function(self)
     lists:setShowMapIcons(DailyGlobalCheck:GetSelectedList(), "worldmap", self:GetChecked())
    end)
 p.showworldmapicons = f
 
 f = ng:New(addonName, "Checkbox", "DGCOptionsPluginMMCheckbox", p)
 f:SetText("|cffFFFFFF"..L["plugin_mapicons_minimap"].."|r")
 f:SetPoint("BOTTOMLEFT", p, "BOTTOMLEFT", 10, 20)
 f:SetScript("OnClick", function(self)
     lists:setShowMapIcons(DailyGlobalCheck:GetSelectedList(), "minimap", self:GetChecked())
    end)
 p.showminimapicons = f

 updatepluginpanel()
 return p
end

local function updatemappanel()
 local O = DailyGlobalCheck_Options
 local of = DGC_Mainframe.panels[4]
 
 if not of then return end

 for k,v in pairs(of.widgets) do
  if v.SetChecked then
   v:SetChecked(O[k] ~= nil and O[k] or O.UI[k])
  elseif v.SetValue then
   v:SetValue(O[k] ~= nil and O[k] or O.UI[k])
  end
 end
end

local function createMapGroup(p, t)
 local grp = ng:New(addonName, "Groupbox", p:GetName().."Group"..t, p)
 grp:SetSize(200, 100)
 grp.text:SetText(string.upper(t))
 
 local O = DailyGlobalCheck_Options.UI
 
 -- enable
 local f = ng:New(addonName, "Checkbox", "DGC"..t.."CheckboxEnable", p)
 f:SetPoint("TOPLEFT", grp, "TOPLEFT", 20, -5)
 f:SetText("|cffFFFFFF"..ENABLE.."|r")
 f:SetScript("OnClick", function(self)
   DailyGlobalCheck_Options["show_"..t.."icons"] = self:GetChecked()
   options:OnOptionChanged("show_"..t.."icons")
   DailyGlobalCheck:Refresh()
  end)
 p.widgets["show_"..t.."icons"] = f

 -- interactable icons
 f = ng:New(addonName, "Checkbox", "DGC"..t.."CheckboxInteract", p)
 f.text:SetText("|cffFFFFFF"..MAKE_INTERACTABLE.."|r")
 f:SetPoint("TOPLEFT", grp, "TOPLEFT", 20, -30)
 f:SetChecked(O[t.."_icon_interact"])
 f:SetScript("OnClick", function(self)
   O[t.."_icon_interact"] = self:GetChecked()
   options:OnOptionChanged(t.."_icon_interact")
   DailyGlobalCheck:Refresh()
  end)
 p.widgets[t.."_icon_interact"] = f

 -- icon size
 f = ng:New(addonName, "SliderH", "DGC"..t.."IconSizeSlider", grp)
 f:SetPoint("TOPLEFT", grp, "TOPLEFT", 20, -60)
 f:SetMinMaxValues(C.mapicons_size_min, C.mapicons_size_max)
 f.onValueChanged = function(self, value)
                     self.preview_icon:SetSize(value, value)
					 self.text_value:SetText(L["size"].." "..AddColor(math.floor(value), "LIGHT_GREEN"))
					 O[t.."_icon_size"] = value
					 options:OnOptionChanged(t.."_icon_size")
                    end

 f.preview_icon = grp:CreateTexture()
 f.preview_icon:SetPoint("LEFT", f, "RIGHT", 20, 0)
 f.preview_icon:SetTexture(dgcicon)
 f:SetValue(O[t.."_icon_size"])
 p.widgets[t.."_icon_size"] = f
 return grp
end

function options.setup_map_panel(p)
 p.widgets = {}

 -- world map
 local grp1 = createMapGroup(p, "worldmap")
 grp1:SetPoint("TOPLEFT", p, "TOPLEFT", 10, -20)
 grp1:SetPoint("BOTTOMRIGHT", p, "TOPRIGHT", -10, -120)
 
 -- minimap
 local grp2 = createMapGroup(p, "minimap")
 grp2:SetPoint("TOPLEFT", grp1, "BOTTOMLEFT", 0, -20)
 grp2:SetPoint("BOTTOMRIGHT", grp1, "BOTTOMRIGHT", 0, -120)
 
 updatemappanel()
end

function options:OnDraw(mode)
 if not DGC_Mainframe then return end

 if not mode then
  updatepluginpanel()
 elseif mode == C.MODE_OPTIONS_PLUGIN then
  updatepluginpanel()
 elseif mode == C.MODE_OPTIONS_MAP then
  updatemappanel()
 end
end

-- Blizzard Interface Panel --
local widgets = {}

-- Daily Global Check (parent)
local function generateMainOptions(p)
--
end

-- UI
local function generateUIOptions(self)
 if not self.initialized then
  self.initialized = true
 end

end

-- Map Icons
local function generateMapOptions(self)
 if not self.initialized then
  
  local function createMapGroup(t)
   local grp = constructors:Groupbox(self, nil, 200, 100, string.upper(t))

   local O = DailyGlobalCheck_Options.UI

   -- interactable worldmap icons
   local f = constructors:Checkbox(grp, 140, MAKE_INTERACTABLE)
   f.title:SetJustifyH("LEFT")
   f:SetPoint("TOPLEFT", grp, "TOPLEFT", 10, -10)
   f:SetChecked(O[t.."_icon_interact"])
   widgets[t.."_icon_interact"] = f

   -- worldmap icon size
   f = constructors:Slider(grp, "DGC"..t.."IconSizeSlider", "", C.mapicons_size_min, C.mapicons_size_max, function(self, value)
                                                                      self.preview_icon:SetSize(value, value)
																	  self.lbl:SetText(L["size"].." "..AddColor(math.floor(value), "LIGHT_GREEN"))
                                                                     end)

   f.preview_icon = self:CreateTexture()
   f.preview_icon:SetPoint("LEFT", f, "RIGHT", 20, 0)
   f.preview_icon:SetTexture(dgcicon)
   f:SetPoint("TOPLEFT", grp, "TOPLEFT", 10, -50)
   f:SetValue(O[t.."_icon_size"])
   widgets[t.."_icon_size"] = f
   return grp
  end

  -- world map
  local grp = createMapGroup("worldmap")
  grp:SetPoint("TOPLEFT", self, "TOPLEFT", 10, -60)
  grp:SetPoint("TOPRIGHT", self, "TOP", -10, -60)
  -- minimap
  grp = createMapGroup("minimap")
  grp:SetPoint("TOPLEFT", self, "TOP", 10, -60)
  grp:SetPoint("TOPRIGHT", self, "TOPRIGHT", -10, -60)
  
  self.initialized = true
 end
end